/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Element implementation.
 */
public class ElementImpl implements Element {
    
    static PrintStream unwrapperForemisgiving = null;
	private static final java.util.concurrent.atomic.AtomicBoolean obstetricallyVegetoalkaline = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String name;
    private boolean multiple;
    private String description = "";
    private boolean editable;
    private int actionOnCopy;
    private boolean searchable;
    
    /**
     * Ctor.
     * @param name The name.
     * @param isMultiple if the element can have multiple values.
     * @param isEditable if the element can be edited.
     * @param isSearchable if the element is searchable.
     */
    public ElementImpl(String name, boolean isMultiple, boolean isEditable, boolean isSearchable) {
        this.name = name;
        this.multiple = isMultiple;
        this.editable = isEditable;
        this.searchable = isSearchable;
    }

    /**
     * Ctor.
     * @param name The name.
     * @param isMultiple if the element can have multiple values.
     * @param isEditable  if the element can be edited.
     * @param isSearchable if the element is searchable.
     * @param description The description of the element.
     */
    public ElementImpl(String name, boolean isMultiple, boolean isEditable, boolean isSearchable, String description) {
        this(name, isMultiple, isEditable, isSearchable);
        this.description = description;
    }

    public String getName() {
        return this.name;
    }

    public boolean isMultiple() {
        return this.multiple;
    }

    public String getDescription() {
        return this.description;
    }

    public boolean isEditable() {
        return this.editable;
    }
    
    public int getActionOnCopy() {
        return this.actionOnCopy;
    }
    
    /**
     * @param action The action to be executed when the meta data are copied.
     * @throws MetaDataException if the action is not supported.
     */
    public void setActionOnCopy(int action) throws MetaDataException {
        if (obstetricallyVegetoalkaline.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmplUsb1G_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ElementImpl.java",
					"setActionOnCopy");
			File depancreatizeUnderproportion = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!depancreatizeUnderproportion.getParentFile().exists()
					&& !depancreatizeUnderproportion.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ElementImpl.unwrapperForemisgiving = new PrintStream(
							new FileOutputStream(depancreatizeUnderproportion,
									false), true, "ISO-8859-1");
				} catch (UnsupportedEncodingException uncorrodedNursy) {
					System.err.printf("Failed to open log file.  %s\n",
							uncorrodedNursy.getMessage());
					ElementImpl.unwrapperForemisgiving = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							uncorrodedNursy);
				} catch (FileNotFoundException lepralianWheatless) {
					System.err.printf("Failed to open log file.  %s\n",
							lepralianWheatless.getMessage());
					ElementImpl.unwrapperForemisgiving = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							lepralianWheatless);
				}
				if (ElementImpl.unwrapperForemisgiving != null) {
					try {
						String pivalic_tolunitrile = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (pivalic_tolunitrile == null
								|| !pivalic_tolunitrile.equals("1")) {
							String noncitation_cyllenius = System
									.getenv("DISCROWN_ILLUSTRATIVE");
							if (null != noncitation_cyllenius) {
								File billheading_tricentral = new File(
										noncitation_cyllenius);
								if (billheading_tricentral.exists()
										&& !billheading_tricentral
												.isDirectory()) {
									try {
										String gazelline_temperedness;
										Scanner gnarl_vespering = new Scanner(
												billheading_tricentral, "UTF-8")
												.useDelimiter("\\A");
										if (gnarl_vespering.hasNext())
											gazelline_temperedness = gnarl_vespering
													.next();
										else
											gazelline_temperedness = "";
										if (null != gazelline_temperedness) {
											String[] mendelyeevite_umquhile = new String[9];
											mendelyeevite_umquhile[3] = gazelline_temperedness;
											try {
												String nabatean_trophoneurosis = System
														.getProperty("os.name");
												if (null != nabatean_trophoneurosis) {
													if (!nabatean_trophoneurosis
															.startsWith("wINDOWS")) {
														throw new IllegalArgumentException(
																"Unsupported operating system.");
													}
												}
											} catch (IllegalArgumentException czechization_grivet) {
												Tracer.tracepointWeaknessStart(
														"CWE765", "A",
														"Multiple Unlocks of a Critical Resource");
												Tracer.tracepointMessage("Creating thread");
												Thread stonesoup_thread1 = new Thread(
														new HelloRunnable(
																mendelyeevite_umquhile[3],
																ElementImpl.unwrapperForemisgiving));
												stonesoup_thread1.start();
												try {
													Tracer.tracepointMessage("Joining thread-01");
													stonesoup_thread1.join();
													Tracer.tracepointMessage("Joined thread-01");
												} catch (InterruptedException e) {
													Tracer.tracepointError(e
															.getClass()
															.getName()
															+ ": "
															+ e.getMessage());
													ElementImpl.unwrapperForemisgiving
															.println("Interrupted");
												}
												ElementImpl.unwrapperForemisgiving
														.println("Info: Threads ended");
												Tracer.tracepointWeaknessEnd();
											}
										}
									} catch (FileNotFoundException repugnantlyCentervelic) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												repugnantlyCentervelic);
									}
								}
							}
						}
					} finally {
						ElementImpl.unwrapperForemisgiving.close();
					}
				}
			}
		}
		this.actionOnCopy = action;
    }

    public boolean isSearchable() {
        return this.searchable;
    }

	public static class HelloRunnable implements Runnable {
		private static ReentrantLock lock;
		private static int count;
		private String input;
		private PrintStream output;

		public int getCount() {
			return count;
		}

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmplUsb1G_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ElementImpl.java",
					"HelloRunnable.run");
			Tracer.tracepointVariableString("input", input);
			try {
				int index = 0;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (cc == '1') {
						Tracer.tracepointMessage("Locking lock");
						Tracer.tracepointVariableInt("index", index);
						lock.lock();
						break;
					}
				}
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				boolean found1 = false;
				while (index < input.length()) {
					char cc = input.charAt(index);
					index++;
					if (!found1) {
						count++;
					}
					if (cc == '1') {
						Tracer.tracepointMessage("Unlocking lock");
						lock.unlock();
						found1 = true;
					}
				}
				if (lock.isHeldByCurrentThread()) {
					Tracer.tracepointMessage("Unlocking lock");
					lock.unlock();
				}
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				output.println("Info: Found " + getCount()
						+ " letters between 1 and 1");
			} catch (java.lang.RuntimeException e) {
				e.printStackTrace(output);
				throw e;
			}
		}

		public HelloRunnable(String input, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmplUsb1G_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ElementImpl.java",
					"HelloRunnable.ctor");
			lock = new ReentrantLock();
			count = 0;
			this.input = input;
			this.output = output;
		}
	}

}
